//
// Created by philipp on 05.02.18.
//

#include "../include/PC2CarLib/CommandReceiver.h"

namespace pc2car
{

CommandReceiver::Ptr
CommandReceiver::create(networking::Networking & net,
                        bool defaultLoggingEnabled,
                        bool defaultPlatoonEnabled,
                        bool defaultRCModeEnabled,
                        float defaultPlatoonSpeed,
                        float defaultInnerPlatoonDistance,
                        float defaultSpeed)
{
    return std::make_shared<CommandReceiver>(
        PrivateTag{},
        net,
        defaultLoggingEnabled,
        defaultPlatoonEnabled,
        defaultRCModeEnabled,
        defaultPlatoonSpeed,
        defaultInnerPlatoonDistance,
        defaultSpeed);
}

CommandReceiver::CommandReceiver(CommandReceiver::PrivateTag,
                                 networking::Networking & net,
                                 bool defaultLoggingEnabled,
                                 bool defaultPlatoonEnabled,
                                 bool defaultRCModeEnabled,
                                 float defaultPlatoonSpeed,
                                 float defaultInnerPlatoonDistance,
                                 float defaultSpeed)
    : loggingEnabled(defaultLoggingEnabled)
      , platoonEnabled(defaultPlatoonEnabled)
      , rcModeEnabled(defaultRCModeEnabled)
      , platoonSpeed(defaultPlatoonSpeed)
      , innerPlatoonDistance(defaultInnerPlatoonDistance)
      , speed(defaultSpeed)
{
    receiver = DatagramReceiver::create(net, PORT, 50);
}

void CommandReceiver::receiveCommands(const CommandReceiver::OnCommandReceivedCallback & handler)
{
    this->handler = handler;
    receiveCommand();
}

void CommandReceiver::stop()
{
    receiver->stop();
}

void CommandReceiver::receiveCommand()
{
    auto self = shared_from_this();
    receiver->asyncReceive(
        networking::time::Duration::max(),
        [self](const auto & error, const auto & message, const auto & host, auto port)
        {
            if (error == networking::error::codes::ABORTED)
            {
                return;
            }
            else if (error)
            {
                self->receiveCommand();
                return;
            }

            try
            {
                using json = nlohmann::json;
                json j = json::parse(message);
                auto code = j.at(jsonNames::COMMAND).get<CommandCode>();
                self->setValue(code, j);
                self->handler(code);
            }
            catch (...)
            { /* Message invalid. */ }

            self->receiveCommand();
        });
}

void CommandReceiver::setValue(CommandCode code, nlohmann::json & j)
{
    using namespace commandCodes;
    auto & jsonVal = j.at(jsonNames::VALUE);

    switch (code)
    {
        case ENABLE_LOGGING:
            loggingEnabled = jsonVal.get<bool>();
            break;

        case ENABLE_PLATOON:
            platoonEnabled = jsonVal.get<bool>();
            break;

        case ENABLE_RC_MODE:
            rcModeEnabled = jsonVal.get<bool>();
            break;

        case SET_PS:
            platoonSpeed = jsonVal.get<platoonProtocol::PlatoonSpeed>();
            break;

        case SET_IPD:
            innerPlatoonDistance = jsonVal.get<platoonProtocol::InnerPlatoonDistance>();
            break;

        case SET_SPEED:
            speed = jsonVal.get<float>();
            break;
    }
}


}